/**********************************************************************************************************************
 * Copyright (c) Prophesee S.A.                                                                                       *
 *                                                                                                                    *
 * Licensed under the Apache License, Version 2.0 (the "License");                                                    *
 * you may not use this file except in compliance with the License.                                                   *
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0                                 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed   *
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                      *
 * See the License for the specific language governing permissions and limitations under the License.                 *
 **********************************************************************************************************************/

#ifndef METAVISION_HAL_GEN31_EVK2_TZ_DEVICE_H
#define METAVISION_HAL_GEN31_EVK2_TZ_DEVICE_H

#include "devices/treuzell/tz_psee_video.h"
#include "devices/treuzell/tz_issd_device.h"
#include "devices/treuzell/tz_regmap_device.h"
#include "facilities/tz_monitoring.h"
#include "devices/treuzell/tz_main_device.h"
#include "devices/common/evk2_system_control.h"

namespace Metavision {

class TzEvk2Gen31 : public TzPseeVideo, public TzIssdDevice, public TemperatureProvider, public IlluminationProvider {
public:
    TzEvk2Gen31(std::shared_ptr<TzLibUSBBoardCommand> cmd, uint32_t dev_id, std::shared_ptr<TzDevice> parent);
    virtual ~TzEvk2Gen31();
    static std::shared_ptr<TzDevice> build(std::shared_ptr<TzLibUSBBoardCommand> cmd, uint32_t dev_id,
                                           std::shared_ptr<TzDevice> parent);
    static bool can_build(std::shared_ptr<TzLibUSBBoardCommand>, uint32_t dev_id);

    virtual void start();
    virtual void stop();
    virtual StreamFormat get_output_format();
    virtual long get_system_id();
    virtual bool set_mode_standalone();
    virtual bool set_mode_master();
    virtual bool set_mode_slave();
    virtual I_DeviceControl::SyncMode get_mode();
    virtual I_HW_Identification::SensorInfo get_sensor_info() {
        return {3, 1};
    }
    long long get_sensor_id();
    virtual int get_temperature();
    virtual int get_illumination();

protected:
    virtual void spawn_facilities(DeviceBuilder &device_builder);

private:
    Evk2SystemControl sys_ctrl_;
    I_DeviceControl::SyncMode sync_mode_;
};

} // namespace Metavision

#endif // METAVISION_HAL_GEN31_EVK2_TZ_DEVICE_H
